@@ -1,6 +1,6 @@
 package SkewHeap;
 # ABSTRACT: A fast heap structure for Perl
-$SkewHeap::VERSION = '0.01';
+$SkewHeap::VERSION = '0.02';
 use strict;
 use warnings;
 use Inline C => 'DATA', optimize => '-O2';
@@ -17,7 +17,7 @@ SkewHeap - A fast heap structure for Perl
 
 =head1 VERSION
 
-version 0.01
+version 0.02
 
 =head1 SYNOPSIS
 
@@ -26,8 +26,7 @@ version 0.01
   my $heap = SkewHeap->new(sub{ $a <=> $b });
   $heap->put(42);
   $heap->put(35);
-  $heap->put(200);
-  $heap->put(62);
+  $heap->put(200, 62);
 
   $heap->top;  # 35
   $heap->size; # 4
@@ -67,7 +66,7 @@ it from the heap.
 
 =head2 put
 
-Inserts a new element into the heap.
+Inserts one or more new elements into the heap.
 
 =head2 take
 
@@ -116,7 +115,6 @@ typedef struct SkewNode {
   struct SkewNode *left;
   struct SkewNode *right;
   SV *value;
-  SV *cmp;
 } skewnode_t;
 
 typedef struct SkewHeap {
@@ -125,13 +123,12 @@ typedef struct SkewHeap {
   SV *cmp;
 } skewheap_t;
 
-skewnode_t* new_node(SV *cmp, SV *value) {
+skewnode_t* new_node(SV *value) {
   skewnode_t *node;
   Newx(node, 1, skewnode_t);
   node->left  = NULL;
   node->right = NULL;
   node->value = newSVsv(value);
-  node->cmp   = cmp;
   return node;
 }
 
@@ -150,7 +147,7 @@ SV* new(const char *class, SV *cmp) {
   Newx(heap, 1, skewheap_t);
   heap->root = NULL;
   heap->size = 0;
-  heap->cmp = cmp;
+  heap->cmp  = newSVsv(cmp);
 
   obj = newSViv((IV) heap);
   ref = newRV_noinc(obj);
@@ -163,6 +160,7 @@ SV* new(const char *class, SV *cmp) {
 void DESTROY(SV *ref) {
   skewheap_t *heap = SKEW(ref);
   if (heap->root != NULL) free_node(heap->root);
+  SvREFCNT_dec(heap->cmp);
   Safefree(heap);
 }
 
@@ -325,19 +323,27 @@ void _merge(skewheap_t *heap, skewnode_t *a, skewnode_t *b) {
   }
 }
 
-int put(SV *ref, SV *value) {
+void put(SV *ref, ...) {
+  Inline_Stack_Vars;
+  Inline_Stack_Reset;
+
   skewheap_t *heap = SKEW(ref);
-  skewnode_t *node = new_node(heap->cmp, value);
+  skewnode_t *node;
+  int i;
+ 
+  for (i = 1; i < Inline_Stack_Items; ++i) {
+    node = new_node(Inline_Stack_Item(i));
+    ++heap->size;
 
-  ++heap->size;
-
-  if (heap->root == NULL) {
-    heap->root = node;
-  } else {
-    _merge(heap, heap->root, node);
+    if (heap->root == NULL) {
+      heap->root = node;
+    } else {
+      _merge(heap, heap->root, node);
+    }
   }
 
-  return 1;
+  Inline_Stack_Push(sv_2mortal(newSViv(heap->size)));
+  Inline_Stack_Done;
 }
 
 void take(SV *ref) {
